@@ -8,6 +8,7 @@ use crate::{
         common::{BakedMaterialShaders, MaterialSignature, MaterialValue, MaterialValueType},
         graph::{node::MaterialGraphNodeId, MaterialGraph},
     },
+    render_target::RenderTargetError,
     resources::resource_mapping::ResourceMapping,
     HasContextResources, ResourceInstanceReference,
 };
@@ -65,7 +66,7 @@ pub enum MaterialError {
         to_value_type: Option<MaterialValueType>,
         param: Option<String>,
     },
-    GraphIsCyclic(MaterialGraphNodeId),
+    GraphIsCyclic(Vec<MaterialGraphNodeId>),
     NoTransferFound(MaterialGraphNodeId),
     CouldNotCompileVertexShader(String),
     CouldNotCompileFragmentShader(String),
@@ -75,6 +76,8 @@ pub enum MaterialError {
     InvalidUniformTypeToSubmit(MaterialValueType),
     CouldNotBuildSubgraphForSignature(MaterialSignature),
     SubgraphInputsDoesNotMatchSignature(HashSet<String>, MaterialSignature),
+    Baking(MaterialGraph, Box<MaterialError>),
+    CouldNotCreateRenderTarget(RenderTargetError),
 }
 
 pub type MaterialInstanceReference = ResourceInstanceReference<MaterialId>;
@@ -898,6 +901,8 @@ macro_rules! material_graph {
             #[allow(unused_mut)]
             let mut ___graph = $crate::material::graph::MaterialGraph::default();
             $(
+                #[deny(clippy::shadow_reuse)]
+                #[warn(clippy::shadow_unrelated)]
                 #[allow(non_snake_case)]
                 let $in_name = ___graph.add_node($crate::material_graph_input! {
                     $( [ $in_shader_type ] )?
@@ -906,6 +911,8 @@ macro_rules! material_graph {
                 });
             )*
             $(
+                #[deny(clippy::shadow_reuse)]
+                #[warn(clippy::shadow_unrelated)]
                 #[allow(non_snake_case)]
                 let $out_name = ___graph.add_node($crate::material_graph_output! {
                     $( [ $out_shader_type ] )? $out_data_type $out_name : $out_value_type
@@ -924,6 +931,8 @@ macro_rules! material_graph {
         }
     };
     ( @statement [ $node:ident = $expression:tt ], $graph:expr, $result:ident ) => {
+        #[deny(clippy::shadow_reuse)]
+        #[warn(clippy::shadow_unrelated)]
         #[allow(non_snake_case)]
         let $node = $crate::material_graph!(@expression $expression, $graph);
     };
